/***************************************************************************
 *   Copyright (C) 2004 by Byoungyoung, La                                 *
 *   la9527@yahoo.co.kr                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "TargzUtil.h"

using namespace MLS;

TargzUtil::TargzUtil(const string& sDir_Filename, bool isZip)
{
	m_Dir = sDir_Filename.substr(0, sDir_Filename.find_last_of("/"));
	m_Filename = sDir_Filename.substr(	sDir_Filename.find_last_of("/"),
					sDir_Filename.length()-sDir_Filename.find_last_of("/"));
					
	m_isZip = isZip;
	
	m_GzipDir = "";
	m_Bzip2Dir = "";
	m_TarDir = "";
	m_sCwd = "";
	
	m_tFileList.clear();
	m_ErrorInfo.clear();
}


TargzUtil::TargzUtil(const string& sDir, const string& sFilename, bool isZip)
{
	m_Dir = sDir;
	m_Filename = sFilename;
	m_isZip = isZip;	
	
	m_GzipDir = "";
	m_Bzip2Dir = "";
	m_TarDir = "";
	m_sCwd = "";
	
	m_tFileList.clear();
	m_ErrorInfo.clear();
}

TargzUtil::~TargzUtil()
{
	File*	pFile = NULL;
	uint	nCount;
	
	BeforeChdir();
	
	for (nCount = 0; nCount < (uint)m_tFileList.size(); nCount++)
	{
		pFile = m_tFileList[nCount];
		delete pFile;
	}
}

// chdir 압축을 풀려면 압축을 풀 디렉토리로 이동해야 된다.
// tar, gzip을 같이 이용하기 때문.
int
TargzUtil::Chdir(void)
{
	char*	cCwd = new char[1024];
	memset(cCwd, 0, 1024);
	if (getcwd(cCwd, 1024) == NULL) 
	{
		delete cCwd;
		throw Exception("GetCwd :: Buffer over flow :: Directory Name is too long..");
		return ERROR;
	}
	
	m_sCwd = cCwd;
	if (chdir(m_sCwd.c_str()) == ERROR) 
	{
		throw Exception("chdir Error");
		delete cCwd;
		return ERROR;
	}
	delete cCwd;
	return SUCCESS;
}

// 이전 디렉토리로 돌아간다.
int
TargzUtil::BeforeChdir(void)
{
	if (m_sCwd == "") return ERROR;

	if (chdir(m_sCwd.c_str()) == ERROR)
	{
		throw Exception("chdir Error");
		return ERROR;
	}
	return SUCCESS;
}

// 파일이 있는지 확인
bool
TargzUtil::IsFileCreate(const string& sFilename)
{
	string sDir = m_Dir + sFilename;
	FILE*	pf = fopen(sDir.c_str(), "rb");
	
	if (!pf) return true;
	fclose(pf);	
	return false;
}

// 확장자를 제외한 기본파일 이름 반환 (Test.tar.gz -> Test.tar)
string
TargzUtil::GetTarFileName(void)
{
	string sFilename = "";
	string::size_type tSize;
	
	tSize = m_Filename.find(".tar");
	if (tSize != string::npos) sFilename = m_Filename.substr(0, tSize+4);
	
	tSize = m_Filename.find(".tgz");
	if (tSize != string::npos) 
	{
		sFilename = m_Filename;
		sFilename.replace(tSize, 4, ".tar");
	}
		
	tSize = m_Filename.find(".tgz");
	if (tSize != string::npos) 
	{
		sFilename = m_Filename;
		sFilename.replace(tSize, 4, ".tar");
	}
	
	LOG("FileName Convert :: %s", sFilename.c_str());
	return sFilename;
}

// 확장자 체크 
int
TargzUtil::FileExtCheck(const char* sExt)
{
	string::size_type tStringSize = m_Filename.find(sExt);
	
	if (tStringSize == string::npos) 
		return ERROR;
	return SUCCESS;
}

// 화일 구분.
int	
TargzUtil::GetFileInfo(void)
{
	if (GetTarFileName() == "") 
	{
		LOG("GetTarFileName NULL");
		return ERROR;
	}
	
	if (FileExtCheck(".tar.gz") == SUCCESS 	|| FileExtCheck(".tgz") == SUCCESS 
	 || FileExtCheck(".tar.z")  == SUCCESS  || FileExtCheck(".tar.Z") == SUCCESS) 
	{
		LOG("m_isFileInfo :: TAR_GZ");
		m_isFileInfo = TAR_GZ;
	}
	else if (FileExtCheck(".tar.bz2") == SUCCESS || FileExtCheck(".tbz") == SUCCESS) 
	{
		LOG("m_isFileInfo :: TAR_BZ");
		m_isFileInfo = TAR_BZ;
	}
	else if (FileExtCheck(".tar") == SUCCESS) 
	{
		LOG("m_isFileInfo :: TAR");
		m_isFileInfo = TAR;
	}
	else if (FileExtCheck(".zip") == SUCCESS) 
	{
		LOG("m_isFileInfo :: ZIP");
		m_isFileInfo = ZIP;
	}
	else 
	{
		LOG("m_isFileInfo :: ERROR");
		return ERROR;
	}
	return SUCCESS;
}

// Command 명령 실행. 
int
TargzUtil::CommandExecute(const string& sCommand)
{
	char	cLine[1024];
	FILE*	pfFile = NULL;
	m_ErrorInfo.clear();
	
	//cout << "Command Execute :: " << sCommand << endl;
	
	pfFile = popen(sCommand.c_str(), "r");
	
	if (pfFile) {
		rewind(pfFile);
		// 줄단위로 데이터 읽음.
		while (fgets(cLine, sizeof(cLine), pfFile))
		{
			// 에러가 나면 m_ErrorInfo로 저장.
			m_ErrorInfo.push_back(cLine);
		}
		pclose(pfFile);
	}
	else
	{
		throw Exception("Don't Execute");
		return ERROR;
	}
	
	if (m_ErrorInfo.size() != 0) return ERROR;	
	return SUCCESS;
}

// 압축 화일의 정보 읽음.
int
TargzUtil::
FileListRead(void)
{
	FILE*    pfFile = NULL;
	string::size_type tStringSize;
	string::size_type tStringSize_tmp_1;
	string::size_type tStringSize_tmp_2;
	string	sCommand;
	string 	sLine;
	string	sToken;
 	string	sTmp;
	int		nCount = 0;
	
	vector<string>	tTmp;
	m_ErrorInfo.clear();
	
	File*	pFileInfo = NULL;
	
	char	cLine[1024];
	memset(&cLine, 0, sizeof(cLine));
	
	if (m_Filename.length() == 0) {
		throw Exception("Error :: filename is NULL");
		return ERROR;
	}
	
	// 파일 확장자 체크
	if (GetFileInfo() == ERROR) 
	{
		//throw Exception("Error :: Filename Info ERROR");
		LOG("file GetFileInfo ERROR :: %s", sCommand.c_str());
		return ERROR;
	}
	
	switch(m_isFileInfo)
	{
		case	TAR_GZ:
			sCommand = m_TarDir + "tar tvfz " + m_Dir + m_Filename;
			break;
		case	TAR_BZ:
			sCommand = m_TarDir + "tar tvfj " + m_Dir + m_Filename;
			break;
		case	TAR:
			sCommand = m_TarDir + "tar tvf " + m_Dir + m_Filename;
			break;
		case	ZIP:
			// 나중에 추가
			//sCommand = "/bin/tar tvfj " + m_Dir + m_Filename;
			throw Exception("Error :: Filename Info ERROR");
			return ERROR;
			break;
		default:
			return ERROR;
	}
	
	//cout << "Command :: " << sCommand << endl;
	LOG("COMMAND :: %s", sCommand.c_str());
	m_tFileList.clear();
	
	// 실행
	pfFile = popen(sCommand.c_str(), "r");

	if (pfFile) 
	{
		rewind(pfFile);
		// -rw-r--r-- root/root       730 2004-01-12 23:15:53 project/smstest/smstest.kdevelop.pcs
		
		// 줄단위로 데이터 읽음.
		while (fgets(cLine, sizeof(cLine), pfFile))
		{
			sLine = cLine;
			
			// 줄바꿈(\n) 은 지운다.
			sLine.erase(sLine.length() - 1, 1);
			
			LOG("COMMAND Data :: %s", sLine.c_str());
			
			tTmp.clear();
			sToken.clear();
			
			tStringSize = sLine.find(" ");
			if (tStringSize == string::npos) 
			{
				tTmp.push_back(sToken);
				break;
			}
			
			sToken = sLine.substr(0, tStringSize);
			
			// 데이터 하나씩 넣음.
			tTmp.push_back(sToken);
			
			// 공백단위로 끈는다. 
			while(1)
			{
				tStringSize_tmp_1 = sLine.find(" ", tStringSize);
				tStringSize_tmp_2 = sLine.find(" ", tStringSize_tmp_1+1);
				
				if (tStringSize_tmp_2 == string::npos)
				{
					sToken = sLine.substr(tStringSize_tmp_1);
					sToken.erase(0, 1);
					tTmp.push_back(sToken);
					break;
				}
				else
				{
					sToken = sLine.substr(tStringSize_tmp_1, tStringSize_tmp_2 - tStringSize_tmp_1);
					if (sToken != " ") 
					{
						sToken.erase(0, 1);
						tTmp.push_back(sToken);
					}
				}
				
				tStringSize = tStringSize_tmp_1 + 1;
			}
			
			sToken.clear();
			
			if (tTmp.size() >= 6)
			{
				// 파일 정보 채움.
				pFileInfo = new File;
				strncpy(pFileInfo->attr, tTmp[0].c_str(), 10);
				pFileInfo->size = atol(tTmp[2].c_str());
				strncpy(pFileInfo->date, (tTmp[3].c_str())+2, 8);
				strncpy(pFileInfo->time, tTmp[4].c_str(), 5);
				
				// '/./' 제거
				tStringSize_tmp_1 = 0;
				
				for(;;)
				{
					tStringSize_tmp_1 = tTmp[5].find("/./", tStringSize_tmp_1);
					if (tStringSize_tmp_1 != string::npos)
					{
						tTmp[5].erase(tStringSize_tmp_1, 2);
					}
					else 
						break;
				}
				
				pFileInfo->FullName = tTmp[5];
				
				// 마지막에 '/' 에 있는지 찾는다. (디렉토리인지 확인)
				tStringSize = tTmp[5].find("/", (pFileInfo->FullName).length() - 1);
				
				if (!strncmp(pFileInfo->attr, "d", 1))
					pFileInfo->bDir = true;
				else
					pFileInfo->bDir = false;
					
				// Fullname에서 name 구분.
				if (tStringSize != string::npos)
				{
					sTmp = tTmp[5].substr(0, tTmp[5].length() - 1);
														
					tStringSize_tmp_1 = sTmp.rfind("/");
					
					if (tStringSize_tmp_1 == string::npos)
					{
						pFileInfo->name = tTmp[5];
					}
					else
					{
						pFileInfo->name = tTmp[5].substr(	tStringSize_tmp_1 + 1, 
													tTmp[5].length() - tStringSize_tmp_1);
					}
				}
				else
				{
					sTmp = tTmp[5];
					tStringSize_tmp_1 = sTmp.rfind("/");
					
					pFileInfo->name = tTmp[5].substr(	tStringSize_tmp_1 + 1, 
												tTmp[5].length() - tStringSize_tmp_1);
				}
				
				m_tFileList.push_back(pFileInfo);
			}
			else
			{
				LOG("COMMAND ERROR :: %s", sLine.c_str());
				m_ErrorInfo.push_back(sLine);
			}
		}
		
		pclose(pfFile);
		
		LOG("COMMAND END :: datasize :: %d", m_tFileList.size());
		
		// 에러정보가 있으면 
		if (m_ErrorInfo.size() != 0) 
		{
			LOG("m_ErrorInfo :: %s", m_ErrorInfo[0].c_str());
			return ERROR;
		}
	}
	else
	{
		throw Exception("Error :: Don't execute");
		return ERROR;
	}
        pFileInfo = new File;
	strncpy(pFileInfo->attr, "dr--r--r--", 10);
	pFileInfo->size = 0;
	strncpy(pFileInfo->date, "--------", 8);
	strncpy(pFileInfo->time, "--:--", 5);
	pFileInfo->name = "..";
	pFileInfo->FullName = "/";
	m_tFileList.push_back(pFileInfo);
								
	return SUCCESS;
}

// 디렉토리 별로 파일 정보를 읽어 옴, Return Data :: pRe_Dir
int
TargzUtil::GetDir_Files(string& s_Dir, vector<MLS::File*>* pRe_Dir)
{
	LOG(" GetDir_Files :: %s", s_Dir.c_str());
	File*	pFile = NULL;
	uint	nCount = 0;
	string::size_type	tSizeType;
	string::size_type	tSizeType_tmp_1;
	string::size_type	tSizeType_tmp_2;
	
	vector<MLS::File*>	tDir_Files;
	
	tDir_Files.clear();
	
	if (s_Dir.length() == 0)
	{
		for (nCount = 0; nCount < (uint)m_tFileList.size(); nCount++)
		{
			pFile = NULL;
			pFile = m_tFileList.at(nCount);
			
			tSizeType = pFile->FullName.find("/");
			
			if (tSizeType == string::npos)
			{
				tDir_Files.push_back(pFile);
				continue;
			}
			
			tSizeType_tmp_1 = pFile->FullName.find("/", tSizeType + s_Dir.length() + 1);
			
			if (tSizeType_tmp_1 == string::npos)
			{
				if (pFile->FullName.substr(pFile->FullName.length() - 1) == "/")
				{
					tDir_Files.push_back(pFile);
				}
			}
		}
	}
	else
	{
		for (nCount = 0; nCount < (uint)m_tFileList.size(); nCount++)
		{
			pFile = m_tFileList.at(nCount);
			tSizeType = pFile->FullName.find(s_Dir);
			if (tSizeType == string::npos) 	continue;
			
            tSizeType_tmp_1 = pFile->FullName.find("/", s_Dir.length() + tSizeType);
			
			// 파일 넣기
			if (tSizeType_tmp_1 == string::npos)
			{
				tDir_Files.push_back(pFile);
				continue;
			}
			
			// 디렉토리 넣기
			if (tSizeType_tmp_1 == pFile->FullName.length() - 1)
			{
				tDir_Files.push_back(pFile);
			}
		}
	}
	
	if (tDir_Files.size() == 0) 
	{
		LOG("tDir_files size :: 0 :: ERROR :: %d", m_tFileList.size());
		return ERROR;
	}
	
	pRe_Dir->clear();
	*pRe_Dir = tDir_Files;
	return SUCCESS;
}

// 디렉토리 별로 파일 정보를 읽어 옴, Return Data :: pRe_Dir
int
TargzUtil::GetDir_AllFiles(string& s_Dir, vector<MLS::File*>* pRe_Dir)
{
	
	File*	pFile = NULL;
	uint	nCount;
	string::size_type	tSizeType;
	pRe_Dir->clear();
	
	for (uint nCount = 0; nCount < (uint)m_tFileList.size(); nCount++)
	{
		pFile = m_tFileList[nCount];
		tSizeType = pFile->FullName.find(s_Dir);
		if (tSizeType == string::npos) 	continue;
		
		pRe_Dir->push_back(pFile);
	}	
	return SUCCESS;
}

// 전체 정보 읽어옴.
int 	
TargzUtil::GetAll_Files(vector<MLS::File*>* rRe_File)
{
	*rRe_File = m_tFileList;
	return SUCCESS;
}

// 압축 풀었을때의 전체 파일 사이즈
uint	TargzUtil::GetAllDataSize(void)
{
	uint	uTotal = 0;
	
	File*	pFile = NULL;
	uint	nCount;
	string::size_type	tSizeType;
	
	for (uint nCount = 0; nCount < (uint)m_tFileList.size(); nCount++)
	{
		pFile = m_tFileList[nCount];
		uTotal = uTotal + pFile->size;
	}
	return uTotal;
}

// 화일하나에 대해 Append나 delete를 한다.
// Append : true, Delete : false
int		
TargzUtil::FileControl(const string& sFullName, int nAppendDel)
{
	string	sCommand = "";	
	
	string	sTarFilename = GetTarFileName();
	
	// 우선 파일이 있는지 확인. 없으면 만듦. :: 있으면 true
	if (IsFileCreate(sTarFilename) == true)
	{
		if (nAppendDel == TAR_DELETE)
		{
			throw Exception("ERROR : '"+ m_Filename +"' file not found... :: TAR_DELETE");
			return ERROR;
		}
		
		sCommand = m_TarDir + "tar cf " + m_Dir + sTarFilename + " " + sFullName;
		if (CommandExecute(sCommand) == ERROR) return ERROR;
	}
	else
	{
		switch(nAppendDel)
		{
			case TAR_APPEND:
				sCommand = m_TarDir + "tar rf " + m_Dir + sTarFilename + " " + sFullName;
				if (CommandExecute(sCommand) == ERROR) return ERROR;
				break;
				
			case TAR_DELETE:
				sCommand = m_TarDir + "tar --delete --file " + m_Dir + sTarFilename + " " + sFullName;
				LOG("DELETE :: %s", sCommand.c_str());
				if (CommandExecute(sCommand) == ERROR) return ERROR;
				break;
		}
	}
	return SUCCESS;
}

// Arg 1 :: 압축할 파일 리스트 
// nAppendDel :: TAR_APPEND, TAR_DELETE 가 있음. 
// File들은 name으로만 판별.
int
TargzUtil::Compress(vector<MLS::File*>& t_FileList, int nAppendDel)
{
	string	sCommand = "";
	
	if (Chdir() == ERROR) return ERROR;
	if (GetFileInfo() == ERROR) return ERROR;
	
	// 우선 파일이 있는지 확인. 있으면 압축을 푼다.
	if (IsFileCreate() == false)
	{
		// tar.gz 파일은 압축을 풀고 Append나 Delete를 해야 한다. 
		switch(m_isFileInfo)
		{
			case	TAR_GZ:
				sCommand = m_GzipDir + "gunzip " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
			case 	TAR_BZ:
				sCommand = m_Bzip2Dir + "bunzip2 " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
		}
	}
		
	string::size_type	tSizeType;
	File*	pFile;
		
	for (uint nCount = 0; nCount < (uint)t_FileList.size(); nCount++)
	{
		pFile = t_FileList.at(nCount);
		if (FileControl(pFile->FullName, nAppendDel) == ERROR) { BeforeChdir(); return ERROR; }
	}	
	
	switch(m_isFileInfo)
	{
		case	TAR_GZ:
			sCommand = m_GzipDir + "gzip " + m_Dir + GetTarFileName();
			if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
			break;
		case 	TAR_BZ:
			sCommand = m_Bzip2Dir + "bzip2 " + m_Dir + GetTarFileName();
			if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
			break;
	}
	
	BeforeChdir(); 
	return SUCCESS;
}

// File이 아닌 파일이름을 string으로
int
TargzUtil::Compress(vector<string>& t_FileList, int nAppendDel)
{
	string	sCommand = "";
	
	if (Chdir() == ERROR) return ERROR;
	if (GetFileInfo() == ERROR) return ERROR;
	
	// 우선 파일이 있는지 확인. 있으면 압축을 푼다.
	if (IsFileCreate() == false)
	{
		// tar.gz 파일은 압축을 풀고 Append나 Delete를 해야 한다. 
		switch(m_isFileInfo)
		{
			case	TAR_GZ:
				sCommand = m_GzipDir + "gunzip " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
			case 	TAR_BZ:
				sCommand = m_Bzip2Dir + "bunzip2 " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
		}
	}
		
	string::size_type	tSizeType;
	string	sFullFileName;
		
	for (uint nCount = 0; nCount < (uint)t_FileList.size(); nCount++)
	{
		sFullFileName = t_FileList.at(nCount);
		if (FileControl(sFullFileName, nAppendDel) == ERROR) { BeforeChdir(); return ERROR; }
	}	
	
	switch(m_isFileInfo)
	{
		case	TAR_GZ:
			sCommand = m_GzipDir + "gzip " + m_Dir + GetTarFileName();
			if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
			break;
		case 	TAR_BZ:
			sCommand = m_Bzip2Dir + "bzip2 " + m_Dir + GetTarFileName();
			if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
			break;
	}
	
	BeforeChdir(); 
	return SUCCESS;
}


int
TargzUtil::Compress(MLS::File* pFile, int nAppendDel)
{
	string	sCommand = "";
	
	if (Chdir() == ERROR) return ERROR;
	if (GetFileInfo() == ERROR) return ERROR;
	
	// 우선 파일이 있는지 확인. 있으면 압축을 푼다.
	if (IsFileCreate() == false)
	{
		// tar.gz 파일은 압축을 풀고 Append나 Delete를 해야 한다.
		switch(m_isFileInfo)
		{
			case	TAR_GZ:
				sCommand = m_GzipDir + "gunzip " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
			case 	TAR_BZ:
				sCommand = m_Bzip2Dir + "bunzip2 " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
		}
	}
		
	if (FileControl(pFile->FullName, nAppendDel) == ERROR) { BeforeChdir(); return ERROR; }
	
	switch(m_isFileInfo)
	{
		case	TAR_GZ:
			sCommand = m_GzipDir + "gzip " + m_Dir + GetTarFileName();
			if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
			break;
		case 	TAR_BZ:
			sCommand = m_Bzip2Dir + "bzip2 " + m_Dir + GetTarFileName();
			if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
			break;
	}
	
	BeforeChdir(); 
	return SUCCESS;
}

// Arg 1 :: 압축할 디렉토리
int
TargzUtil::Compress(const string& s_Dir)
{
	string	sCommand = "";
	
	if (Chdir() == ERROR) return ERROR;	
	if (GetFileInfo() == ERROR) return ERROR;
	
	// 우선 파일이 있는지 확인. 있으면 압축을 푼다.
	if (IsFileCreate() == false)
	{
		// tar.gz 파일은 압축을 풀고 Append나 Delete를 해야 한다. 
		switch(m_isFileInfo)
		{
			case	TAR_GZ:
				sCommand = m_GzipDir + "gunzip " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
			case 	TAR_BZ:
				sCommand = m_Bzip2Dir + "bunzip2 " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
		}
		
		sCommand = m_TarDir + "tar rf " + m_Dir + GetTarFileName() + " " + s_Dir; // Append Directory
	}
	else
	{
		sCommand = m_TarDir + "tar cf " + m_Dir + GetTarFileName() + " " + s_Dir; // create
	}
	
	if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
	
	switch(m_isFileInfo)
	{
		case	TAR_GZ:
			sCommand = m_GzipDir + "gzip " + m_Dir + GetTarFileName();
			if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
			break;
		case 	TAR_BZ:
			sCommand = m_Bzip2Dir + "bzip2 " + m_Dir + GetTarFileName();
			if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
			break;
	}
	
	BeforeChdir();
	return SUCCESS;
}

// Arg 1 :: 압축풀 디렉토리
int
TargzUtil::Uncompress(const string& s_Dir)
{
	string	sCommand = "";
	string	sZip = "";
	
	if (Chdir() == ERROR) return ERROR;	
	if (GetFileInfo() == ERROR) { BeforeChdir(); return ERROR; }
	
	// 우선 파일이 있는지 확인. 있으면 압축을 푼다.
	if (IsFileCreate() == false)
	{
		// tar.gz 파일은 압축을 풀고 Append나 Delete를 해야 한다. 
		switch(m_isFileInfo)
		{
			case	TAR_GZ:
				sCommand = m_GzipDir + "gunzip " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
			case 	TAR_BZ:
				sCommand = m_Bzip2Dir + "bunzip2 " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
		}
		
		// 디렉토리 위치를 정해주지 않으면 현재 디렉토리에 풀리게 된다.
		if (s_Dir == "") 
			sCommand = m_TarDir + "tar xf" + sZip + " " + m_Dir + GetTarFileName();
		else
			sCommand = m_TarDir + "tar xf" + sZip + " " + m_Dir + GetTarFileName() + " -C " + s_Dir; // extract Directory
		
		if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
		
		switch(m_isFileInfo)
		{
			case	TAR_GZ:
				sCommand = m_GzipDir + "gzip " + m_Dir + GetTarFileName();
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
			case 	TAR_BZ:
				sCommand = m_Bzip2Dir + "bzip2 " + m_Dir + GetTarFileName();
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
		}
	}
	else
	{
		BeforeChdir();
		throw Exception("Error :: '" + m_Dir + m_Filename + "' File not found");
		return ERROR;
	}
	
	return SUCCESS;
}

// Arg 1 :: 압축풀 파일 리스트 
int
TargzUtil::Uncompress(vector<MLS::File*>& t_FileList, const string& sTargetDir)
{
	string	sCommand = "";
	string	sZip = "";
	
	if (Chdir() == ERROR) return ERROR;
	if (GetFileInfo() == ERROR) { BeforeChdir(); return ERROR; }
	
	// 우선 파일이 있는지 확인. 있으면 압축을 푼다.
	if (IsFileCreate() == false)
	{
		// tar.gz 파일은 압축을 풀고 Append나 Delete를 해야 한다. 
		switch(m_isFileInfo)
		{
			case	TAR_GZ:
				sCommand = m_GzipDir + "gunzip " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
			case 	TAR_BZ:
				sCommand = m_Bzip2Dir + "bunzip2 " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
		}
		
		string::size_type	tSizeType;
		File*	pFile;
		
		for (uint nCount = 0; nCount < (uint)t_FileList.size(); nCount++)
		{
			pFile = t_FileList.at(nCount);
			
			if (sTargetDir.length() == 0)
			{
				sCommand = 	m_TarDir + "tar xf " + m_Dir + GetTarFileName() + 
							" -C " + sTargetDir + " " + pFile->FullName; // extract file
			}
			else
			{
				sCommand = 	m_TarDir + "tar xf " + m_Dir + GetTarFileName() + 
							" " + pFile->FullName; // extract file
			}
						
			// 에러무시
			if (CommandExecute(sCommand) == ERROR) continue;
		}
		
		switch(m_isFileInfo)
		{
			case	TAR_GZ:
				sCommand = m_GzipDir + "gzip " + m_Dir + GetTarFileName();
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
			case 	TAR_BZ:
				sCommand = m_Bzip2Dir + "bzip2 " + m_Dir + GetTarFileName();
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
		}
	}
	else
	{
		BeforeChdir();
		throw Exception("Error :: '" + m_Dir + m_Filename + "' File not found");
		return ERROR;
	}
	BeforeChdir();
	return SUCCESS;
}

// Arg 1 :: String 방식.
int
TargzUtil::Uncompress(vector<string>& t_FileList, const string& sTargetDir)
{
	string	sCommand = "";
	string	sZip = "";
	
	if (Chdir() == ERROR) return ERROR;
	if (GetFileInfo() == ERROR) { BeforeChdir(); return ERROR; }
	
	// 우선 파일이 있는지 확인. 있으면 압축을 푼다.
	if (IsFileCreate() == false)
	{
		// tar.gz 파일은 압축을 풀고 Append나 Delete를 해야 한다. 
		switch(m_isFileInfo)
		{
			case	TAR_GZ:
				sCommand = m_GzipDir + "gunzip " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
			case 	TAR_BZ:
				sCommand = m_Bzip2Dir + "bunzip2 " + m_Dir + m_Filename;
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
		}
		
		string::size_type	tSizeType;
		string	sFullFileName;
		
		for (uint nCount = 0; nCount < (uint)t_FileList.size(); nCount++)
		{
			sFullFileName = t_FileList.at(nCount);
			
			if (sTargetDir.length() == 0)
			{
				sCommand = 	m_TarDir + "tar xf " + m_Dir + GetTarFileName() + 
							" " + sFullFileName; // extract file
			}
			else
			{
				sCommand = 	m_TarDir + "tar xf " + m_Dir + GetTarFileName() + 
							" -C " + sTargetDir + " " + sFullFileName; // extract file
			}
			
			LOG("COMMAND :: %s", sCommand.c_str());
						
			// 에러무시
			if (CommandExecute(sCommand) == ERROR) continue;
		}
		
		switch(m_isFileInfo)
		{
			case	TAR_GZ:
				sCommand = m_GzipDir + "gzip " + m_Dir + GetTarFileName();
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
			case 	TAR_BZ:
				sCommand = m_Bzip2Dir + "bzip2 " + m_Dir + GetTarFileName();
				if (CommandExecute(sCommand) == ERROR) { BeforeChdir(); return ERROR; }
				break;
		}
	}
	else
	{
		BeforeChdir();
		throw Exception("Error :: '" + m_Dir + m_Filename + "' File not found");
		return ERROR;
	}
	BeforeChdir();
	return SUCCESS;
}
