#ifndef DRAWUTIL_H
#define DRAWUTIL_H

#include <ncurses.h>

#define COLOR(fg, bg) COLOR_PAIR(((fg)<<3)+(bg))

namespace MLS {

struct color_entry
{
	int font, back;
	
	color_entry() : font(0), back(0) {}
	
	color_entry(const std::string &line)
	{
		std::string::size_type p = line.find(',');
		
		if (p != std::string::npos)
		{
			font = atoi(line.substr(0, p).c_str());
			back = atoi(line.substr(p+1).c_str());
		}
		else 
		{
			back = 0;
			font = atoi(line.c_str());
		}
		
	}
	
	bool operator == (color_entry &t)
	{
		return font==t.font && back==t.back;
	}
	
	bool operator < (color_entry &t)
	{
		return font*16+back > t.font*16+t.back;
	}
};

inline void setcol(int font, int back, WINDOW *win=stdscr)
{
	if ( font >=8 ) wattron(win, A_BOLD);
	if ( back >=8 ) wattron(win, A_BLINK);
	
	wattron(win, COLOR_PAIR( ((font%8) << 3) + (back%8)));
}

inline void setcol(color_entry &p, WINDOW *win=stdscr)
{
	if ( p.font >=8 ) wattron(win, A_BOLD);
	if ( p.back >=8 ) wattron(win, A_BLINK);
	
	wattron(win, COLOR_PAIR( ((p.font%8) << 3) + (p.back%8)));
}

inline void setrcol(color_entry &p, WINDOW *win=stdscr)
{
	if ( p.font >=8 ) wattron(win, A_BLINK);
	if ( p.back >=8 ) wattron(win, A_BOLD);
	
	wattron(win, COLOR_PAIR( ((p.back%8) << 3) + (p.font%8)));
}


inline void unsetcol(WINDOW *win=stdscr)
{
	wattroff(win, A_BOLD | A_BLINK);	
}

}; // namespace 
#endif
