/***************************************************************************
 *   Copyright (C) 2004 by Byoungyoung, La                                 *
 *   la9527@yahoo.co.kr                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ZipReader.h"
 
using namespace MLS;

ZipReader::ZipReader()
{
	m_pTargzUtil = NULL;
	
	m_sDir = "";
	m_sFilename = "";
	
	m_currentPath = "";
	m_CurrentData = 0;
	
	m_ZipCurrentDir = "";
	m_Filelist.clear();
	
	m_Count = 0;
}

ZipReader::~ZipReader()
{
	if (m_pTargzUtil) delete m_pTargzUtil;
}

bool ZipReader::isRoot(void)
{
	if (m_currentPath.length() == 0) return true;
	return false;
}

// file reader
bool ZipReader::Init(const string &path)
{
	if (m_sFilename == "") return false;
	
	m_CurrentData = 0;
	
	try
	{
		string sPath = path;
		m_ZipCurrentDir = path;
		
		if (m_Count == 0)
		{
			m_pTargzUtil = new TargzUtil(m_sDir, m_sFilename, 0);
			
			if (m_pTargzUtil->FileListRead() == SUCCESS)
			{
				LOG("FileListRead success...");
				
				if (m_pTargzUtil->GetDir_Files(sPath, &m_Filelist) == ERROR)
				{
					LOG("File Count :: %d", m_Count);
					return false;
				}
			}
			else
			{
				LOG("FileListRead failure...");
				return false;
			}
		}
		else
		{
			if (m_pTargzUtil->GetDir_Files(sPath, &m_Filelist) == ERROR)
			{
				LOG("File SIZE :: %d",  m_Filelist.size());
				return false;
			}
		}
	}
	catch(Exception& ex)
	{
		LOG("Exception :: %s", (char*)ex);
		return false;
	}
	
	m_Count++;
	
	LOG("File List :: %d", m_Filelist.size());
	return true;
}

bool ZipReader::Next(void)
{
	LOG("m_CurrentData :: %d, %d", m_CurrentData, m_Filelist.size());
	
	if (m_CurrentData < m_Filelist.size())
	{
		m_CurrentData++;
		return true;
	}
	return false;
}

bool ZipReader::GetInfo(File& tFile)
{
	File*	pFile;
	
	if (m_CurrentData-1 < m_Filelist.size())
	{
		pFile = m_Filelist.at(m_CurrentData-1);
		tFile.clear();
		
		if (m_ZipCurrentDir == pFile->FullName)
			tFile.name = "..";
		else
			tFile.name = pFile->name;
		
		tFile.FullName = pFile->FullName;
		tFile.bDir = pFile->bDir;
		tFile.color = pFile->color;
		strncpy(tFile.date, pFile->date, 9);
		strncpy(tFile.attr, pFile->attr, 11);
		strncpy(tFile.time, pFile->time, 6);
		tFile.selected = pFile->selected;
		tFile.size = pFile->size;
		tFile.mtime = pFile->mtime;
		
		LOG("%s, %s, %d, %s, %s, %s, %ld", 	tFile.name.c_str(), 
										tFile.FullName.c_str(),
										tFile.bDir,
										tFile.date,
										tFile.time,
										tFile.attr,
										tFile.size);
	}
	else
		return false;
	return true;
}

//    Ǯ 
int ZipReader::Copy(vector<string>& tCpyData, string& sTarget)
{
	if (m_pTargzUtil->Uncompress(tCpyData, sTarget) == ERROR)
	{
		return ERROR;
	}
	return SUCCESS;
}

//   
int ZipReader::Remove(vector<string>& tRemoveData)
{
	if (m_pTargzUtil->Compress(tRemoveData, TAR_DELETE) == ERROR)
	{
		return ERROR;
	}
	return SUCCESS;
}

void ZipReader::Destroy(void)
{
	if (m_pTargzUtil) delete m_pTargzUtil; m_pTargzUtil = NULL;
	
	m_sDir = "";
	m_sFilename = "";
	
	m_currentPath = "";
	m_CurrentData = 0;
	
	m_ZipCurrentDir = "";
	m_Filelist.clear();
	
	m_Count = 0;
}

