#include <pwd.h>
#include <unistd.h>
#include <sys/stat.h>
#include "define.h"

#include <map>
#include <fstream>

#include "mls.h"
#include "conf.h"
#include "strutil.h"
#include "StringTokenizer.h"

using namespace std;
using namespace strutil;
using namespace MLS;

Config MLS::conf;

Config::Config()
{
	set_default();
		
	if (load((cfghome + "mls.cfg").c_str())==1) 
	{
		fprintf (stderr, "Fail to load mls.cfg.");
		exit(1);
	}
	
	if (load_colorset((cfghome + colorset).c_str())==1)
	{
		fprintf (stderr, "Colorset file is not found");
		exit(1);
	}
	
	// 후처리
	
	if (mcdroot.empty()) mcdroot = '/';	
	else if (mcdroot[mcdroot.size()-1] != '/') mcdroot += '/';	
}

Config::~Config()
{
}

void Config::set_default()
{// default를 만듭니다..
	struct stat 	  stat_bufs;	
	// home 디렉토리 지정
	struct passwd *pw = getpwuid(getuid()); // getuid이냐 geteuid냐...;
	home  = pw->pw_dir;
	home += '/';
	
	cfghome = home + ".mls/";
	
	if (lstat (cfghome.c_str(), &stat_bufs) == -1)
	{
		fprintf (stderr, "MLS : making mls configuration directory...");
		if (mkdir(cfghome.c_str(), 0700) == -1) 
		{
			fprintf (stderr, "MLS : making directory operation failed...");
			exit(-1);
		}		
	}	
	
	// 모든 엔트리를 초기화 시킨다.
	
	// 펑션 테이블 클리어	
	for (int t=0; t<12; ++t) Func[t].clear();	
	
	// 문자형	
	colorset = "mls.col";
	date_type = "%y-%m-%d";
	extern_viewer = "vi";
	extern_editor = "vi";
	mcdroot = home;
	
	// On/Off 형	
	FixedRow = false;
	DirSort  = true;
	ShowHidden  = false;
	McdSort  = true;
		
	// 정수형	
	sort   = 1;
	column = 1;	
	AutoDrawmode = true;	
	
}

int Config::load(const char *file)
{
	char line[1024];	
	string::size_type p;
	
	map<string, string> cmd;
	string l, var, val;
		
	FILE *fp=fopen(file, "rb");
	
	if (!fp) return 1;
	
	while(!feof(fp))
	{
		line[0]=0;
		fgets(line, 1024, fp);
						
		if (line[0] == '#') continue;
		
		l=line;
		
		// = 을 찾습니다.
		p = l.find('=');		
				
		if (p == string::npos) continue;
		
		var = chop(l.substr(0, p));
		val = chop(l.substr(p+1));
		
		if (val.empty()) continue;
		
		if (!var.compare(0, 3, "ext"))
		{
			StringTokenizer st(getbetween(var, '[', ']'), "; ");
				
			Action pr;
			pr.addEntry(val);
						
			while (st.Next())
				ext_prog[tolower(st.Get())] = pr;			
		}
		else cmd[tolower(var)]= val; // 나머지들..
	}
	
	fclose(fp);
	
	map<string,string>::iterator i, end = cmd.end();
	
	set_default();
	
	// 펑션키	
	if ((i = cmd.find("funcf1")) != end) Func[0].addEntry((*i).second);
	if ((i = cmd.find("funcf2")) != end) Func[1].addEntry((*i).second);
	if ((i = cmd.find("funcf3")) != end) Func[2].addEntry((*i).second);
	if ((i = cmd.find("funcf4")) != end) Func[3].addEntry((*i).second);
	if ((i = cmd.find("funcf5")) != end) Func[4].addEntry((*i).second);
	if ((i = cmd.find("funcf6")) != end) Func[5].addEntry((*i).second);
	if ((i = cmd.find("funcf7")) != end) Func[6].addEntry((*i).second);
	if ((i = cmd.find("funcf8")) != end) Func[7].addEntry((*i).second);
	if ((i = cmd.find("funcf9")) != end) Func[8].addEntry((*i).second);
	if ((i = cmd.find("funcf10"))!= end) Func[9].addEntry((*i).second);
	if ((i = cmd.find("funcf11"))!= end) Func[10].addEntry((*i).second);
	if ((i = cmd.find("funcf12"))!= end) Func[11].addEntry((*i).second);
	
	// 문자형	
	if ((i = cmd.find("colorset")) != end )  colorset = (*i).second;
	if ((i = cmd.find("date_type")) != end ) date_type = (*i).second;
	if ((i = cmd.find("extviewer")) != end ) extern_viewer = (*i).second;
	if ((i = cmd.find("exteditor")) != end ) extern_editor = (*i).second;	
	if ((i = cmd.find("mcdroot")) != end ) mcdroot = (*i).second;	
	
	// On/Off 형	
	if ((i = cmd.find("fixedrow")) != end ) 
		FixedRow = (tolower((*i).second) == "on");
	if ((i = cmd.find("dirsort")) != end )
		DirSort  = (tolower((*i).second) == "on");
	if ((i = cmd.find("showhidden")) != end )
		ShowHidden  = (tolower((*i).second) == "on");
	if ((i = cmd.find("mcdsort")) != end )
		McdSort  = (tolower((*i).second) == "on");
		
	// 정수형	
	if ((i = cmd.find("sort")) != end )   sort = atoi((*i).second.c_str());
	if ((i = cmd.find("column")) != end ) column = atoi((*i).second.c_str());	
	
	AutoDrawmode = column ? false : true;
	
	return 0;
}

int Config::load_colorset(const char *file)
{
	map<string, color_entry> color_set;	
	string line;
	
	ifstream in(file);
	
	if (!in.is_open()) return 1;
		
	while(!in.eof())
	{		
		getline(in, line);
		
		// 주석처리
		if (line[0] == '#') continue;
		
		// = 을 찾는다		
		string::size_type p = line.find('=');
		
		if (p == string::npos) continue;
		
		string var = tolower(chop(line.substr(0, p)));
		string val = tolower(chop(line.substr(p+1)));	
		
		// mask[]
		if (!var.compare(0, 4, "mask"))
			mask_col[strtol(getbetween(var, '[', ']').c_str(), NULL, 8)] = color_entry(val);
		else if(!var.compare(0, 4, "name"))
		{
			StringTokenizer st(getbetween(var, '[', ']'), ";");
			
			color_entry entry(val);
			
			while(st.Next()) name_col[st.Get()] = entry;
		}		
		// ext[]		
		else if (!var.compare(0, 3, "ext"))
		{
			StringTokenizer st(getbetween(var, '[', ']'), "; ");
			
			color_entry entry(val);
			
			while(st.Next()) ext_col[st.Get()] = entry;
		}
		// 이외것들은 컬러셋
		else color_set[var] = color_entry(val);
	}
	
	MCD = color_set["mcd"];
	MCDstat = color_set["mcdstat"];
	Dir = color_set["dir"];
	Drive = color_set["drive"];
	Line = color_set["line"]; 
	FuncN = color_set["funcn"];
	FuncA = color_set["funca"];
	StatN = color_set["statn"];
	StatA = color_set["stata"];
	Else = color_set["else"];
	Help = color_set["help"];
	Quick = color_set["quick"];
	QuickMCD = color_set["quickmcd"];
	Dialog = color_set["dialog"];
	
	return 0;
}
