#include "conf.h"
#include "dirreader.h"

using namespace std;
using namespace MLS;

DirReader::DirReader()
{
	m_handle = NULL;
	m_lpath = new char[FILENAME_MAX];
}

bool DirReader::isRoot()
{
	return (m_path == "/");		
}

bool DirReader::Init(const string &path)
{
	if (chdir(path.c_str())==-1) return false;
	if ((m_handle = opendir("."))==NULL) return false;
	
	m_path = path;
			
	return true;
}

bool DirReader::Next()
{
	if (!m_handle) return false;
	m_pEntry = readdir(m_handle);	
	
	if (!m_pEntry) return false;
	
	return true;
}

bool DirReader::GetInfo(File &file)
{
	bool bLink;
	int  n;
	
	if (lstat(m_pEntry->d_name, &m_statbuf)==-1) return false;
	
	if (S_ISLNK(m_statbuf.st_mode))
	{	
		if ((n = readlink(m_pEntry->d_name, m_lpath, FILENAME_MAX)) == -1) return false;
		m_lpath[n] = '\0';
		
		if (lstat(m_lpath, &m_statbuf) == -1) return false;
		bLink = true;
	}
	else bLink = false;

	
	file.name = m_pEntry->d_name;
	
	if (S_ISDIR(m_statbuf.st_mode)) file.bDir = true;
	else file.bDir = false;
	
	// zip ߱ ؼ '.'  '-'  ٲ. 
	if (bLink)
		file.attr[0]= 'l';
	else 
		file.attr[0]= file.bDir ? 'd' : '-';
		
	file.attr[1]= S_IRUSR & m_statbuf.st_mode ? 'r' : '-';
	file.attr[2]= S_IWUSR & m_statbuf.st_mode ? 'w' : '-';
	file.attr[3]= S_IXUSR & m_statbuf.st_mode ? 'x' : '-';
	file.attr[4]= S_IRGRP & m_statbuf.st_mode ? 'r' : '-';
	file.attr[5]= S_IWGRP & m_statbuf.st_mode ? 'w' : '-';
	file.attr[6]= S_IXGRP & m_statbuf.st_mode ? 'x' : '-';
	file.attr[7]= S_IROTH & m_statbuf.st_mode ? 'r' : '-';
	file.attr[8]= S_IWOTH & m_statbuf.st_mode ? 'w' : '-';
	file.attr[9]= S_IXOTH & m_statbuf.st_mode ? 'x' : '-';
	file.attr[10]='\0';
	
	file.size = m_statbuf.st_size;
	file.mtime = m_statbuf.st_mtime;		
	
	strftime(file.date, 9, conf.date_type.c_str(), localtime(&m_statbuf.st_mtime));
	strftime(file.time, 6, "%H:%M", localtime(&m_statbuf.st_mtime));
	
	
	return true;	
}

void DirReader::Destroy()
{
	closedir(m_handle);
	m_handle = NULL;
	delete m_lpath;
}
