#include <ncurses.h>
#include <vector>
#include <string>
#include <sys/stat.h>

#include "mcd.h"
#include "conf.h"
#include "dialog.h"
#include "strutil.h"
#include "keydef.h"

using namespace std;
using namespace MLS;
using namespace strutil;

Mcd::Mcd(const char *_title)
{
	title = _title;
	root = NULL;
	sRow = 0;
	sCol = 0;
	infofile = conf.cfghome +"treeinfo";
	code = "MLS1";
	
	if (load()==1)
	{
		struct stat stat_bufs;	
		lstat(conf.mcdroot.c_str(), &stat_bufs);
	
		root = new dir(conf.mcdroot, NULL, stat_bufs.st_nlink);
		
	}
}

Mcd::~Mcd()
{
	save();
	if (root)
	for (pDirIterator p = order.begin(); p!=order.end(); ++p)
		delete *p;
}

string &
Mcd::dir::path()
{
	static pDirContainer st;
	static std::string p;
	dir *pNode=this;
	
	p.clear();
	st.clear();
	
	while(pNode->parent != NULL)
	{
		st.push_back(pNode);
		pNode= pNode->parent;
	}
	
	p = pNode->name;
	
	while(!st.empty())
	{
		p += st.back()->name;
		p += '/';
		st.pop_back();				
	}
	return p;
}

int 
Mcd::Search(const string &str)
{
	pDirIterator i=order.begin(), end=order.end();	
	string p=tolower(str);
	
	while(i != end)
	{
		Mcd::dir &rd = *(*i++);
		if ( rd.name.size() < str.size() ) continue;
		if ( tolower( rd.name.substr(0, str.size())) == p) return rd.index;	
		
	}
	// 없으면
	return -1;
}


bool Mcd::proc()
{
	int key, bkey;
	bool bSearch = false;
	
	bExit = false;	
	
	if (cur==order.end()) cur = order.begin();
	
	do
	{
		draw();		
		key = getch();
		
		if (!bSearch && ( ('A' <= key && key <= 'z') || key == '.' ) ) bSearch = true;		
		if (bSearch &&  ( (32<= key && key <=126) || key == KEY_BS ) )
		{
			int tmp;
			// search mode;
			// 뒷부분에 key를 넣고
			
			if (key == KEY_BS) 
			{
				strSearch.erase(strSearch.size()-1, 1);
				if (strSearch.empty()) {bSearch = false; continue;}
			}
			else strSearch += (char) key;
			
			// strSearch로 시작하는 문자를 찾는다..						
			if ((tmp = Search(strSearch))!= -1) cur = order.begin() + tmp;
			else
			{
				strSearch = (char) key;
				// 없으면 처음에 넣고 다시 찾는다.
				
				if ((tmp = Search(strSearch))!= -1) cur = order.begin() + tmp;
				// 없으면 말구..
				else {bSearch = false;strSearch.clear();}
			}
			continue;
		}
		
		bSearch = false;
		strSearch.clear();		
		
		switch(key)
		{
		case 13:
			return true;
			break;
		case KEY_UP:
			up();
			break;
			
		case KEY_DOWN:			
			down();			
			break;	
			
		case KEY_LEFT:
			left();	
			break;
		case KEY_RIGHT:
			right();			
			break;
			
		case '+':
		{
			string p = (*cur)->path();
			scan(*cur);
			setCur(p);
			break;
		}
			
		case '=':
		{
			string p = (*cur)->path();
			scan(*cur, false);
			setCur(p);
			break;
		}
		
		case '-':
		{
			string p = (*cur)->path();
			destroy(*cur);
			setCur(p);
			break;
		}
		/*
		case KEY_F(2):
		{
			int p= YNBox("Rescan directory now?", YN_Y, COLOR_MAGENTA);			
			if (p==YN_N || p==-1) break;
			
			string pa = (*cur)->path();						
			rescan();
			setCur(pa);
			break;
		}*/
			
		case 27:
			bkey = getch();
			switch(bkey)
			{
			case 27:
			case 'X':
			case 'x':
				return false; // 그냥 끝낸것임.. 디렉토리..;;
				break;
			}
			break;
		
		}		
	}while(!bExit);
	
	return true;
}
