#include <ncurses.h>

#include <algorithm>
#include <functional>

#include "menu.h"
#include "conf.h"
#include "drawutil.h"
#include "mlslocale.h"

#include "panel.h"

using namespace std;
using namespace MLS;

Menu::Menu()
{
	num = 0;
	win = NULL;
}

Menu::~Menu()
{
	if (win) delwin(win);
}

void Menu::Draw()
{	
	if (!win) return;
	
	setcol(conf.FuncN, win);	
	wmove(win, 0, 0);
	whline(win,' ', COLS);		
	for (int t=0; t<item.size(); t++)
		mvwprintw(win, 0, item[t].GetLocation() , "%-11s", item[t].GetName().c_str());	
	unsetcol(win);
	
	setcol(conf.FuncA, win);	
	for (int t=0; t<item.size(); t++)
		mvwaddch(win, 0, item[t].GetLocation(), item[t].GetName()[0]);	
	unsetcol(win);
	
	setcol(7, 0, win);	
	mvwprintw(win, 0, item[num].GetLocation()-1 , " %s ", item[num].GetName().c_str());
	unsetcol(win);
	
	setcol(11, 0, win);
	mvwaddch(win, 0, item[num].GetLocation(), item[num].GetName()[0]);	
	
	wrefresh(win);
	
	item[num].Draw();	
}

void Menu::Process(Panel &rp)
{
	int key;
	bool bRef=true;
	
	touchline(stdscr, 0, LINES);
	
	while(1)
	{
		// 필요할 때만 화면 원상복귀			
		if (bRef)
		{
			wtouchln(stdscr, 0, LINES, 1);
			wrefresh(stdscr);
			bRef=false;
		}		
		Draw();
		
		key = getch();
		
		if ('`' == key) break;
		
		if (13 == key)
		{
			wtouchln(stdscr, 0, LINES, 1);
			wrefresh(stdscr);
						
			rp.Command(item[num].GetCmd());
			break;			
		}
		
		if (KEY_LEFT == key)
		{
			num = (num+ item.size() -1) % item.size();
			bRef = true;
		}
		if (KEY_RIGHT == key) 
		{
			num = (num + 1) % item.size();		
			bRef = true;
		}
		
		if (KEY_UP == key)	item[num].CursorUp();
		if (KEY_DOWN == key) item[num].CursorDown();
		
	}
}

void Menu::Create()
{
	MenuCategory mls("Mls");
		mls.AddItem("About", "<about>");
		mls.AddItem("Help", "<help>");
		mls.AddItem("", "");
		mls.AddItem("Quit Mls", "<exit>");
	AddItem(mls);
	
	MenuCategory run("Run");
		run.AddItem("Dummy1", "");
	AddItem(run);
	
	MenuCategory files("File");
		files.AddItem("Refresh", "<refresh>");
		files.AddItem("Copy", "<copy>");
		files.AddItem("Move", "");
		files.AddItem("Rename", "<rename>");
		files.AddItem("Delete", "");
		files.AddItem("Chmod", "");
		files.AddItem("", "");
		files.AddItem("View", "<view>");
		files.AddItem("Edit", "<edit>");
		files.AddItem("FileFind", "");
		files.AddItem("", "");
		files.AddItem("Select all", "");
		files.AddItem("Invert select", "");
		files.AddItem("Select group", "");
	AddItem(files);
	
	MenuCategory dire("Directory");
		dire.AddItem("Mcd", "<mcd>");
		dire.AddItem("Qcd", "");
		dire.AddItem("", "");
		dire.AddItem("Make", "<mkdir>");
		dire.AddItem("Properties", "");
		
	AddItem(dire);
	
	MenuCategory arc("Archive");
		arc.AddItem("Into the Archive", "");
	AddItem(arc);
	
	MenuCategory op("Option");
		op.AddItem("Edit config file", "<editconfig>");
		op.AddItem("Edit colorset file", "<editcolorset>");		
	AddItem(op);
	
	if (win) delwin(win);	
	win = newwin(1, COLS, 0, 0);	
	
	CreateWindow();
}

void Menu::CreateWindow()
{
	for_each(item.begin(), item.end(), mem_fun_ref(&MenuCategory::Create));
}

void MenuCategory::Create()
{
	if (win) delwin(win);
	win = newwin(item.size() + 2, 24, 1, location-3);	
}

void MenuCategory::Draw()
{
	if (!win) return;
	
	// ----- 긋기		
	setcol(conf.FuncN, win);	
	wmove(win, 0, 0);
	waddch(win, ' ');
	waddch(win, ULCORNER);
	whline(win, HLINE, 20);
	mvwaddch(win, 0, 22, URCORNER);
	waddch(win, ' ');
			
	for(int c=0; c<item.size(); c++)
	{
		wmove(win, c+1, 0);
		waddch(win, ' ');
		waddch(win, VLINE);
		if (item[c].GetName().empty())
		{// 빈 문자열은 줄긋기
			unsetcol(win);
			setcol(0, conf.FuncN.back, win);
			whline(win, HLINE, 20);
			wmove(win, c+1, 22);			
			unsetcol(win);
			setcol(conf.FuncN, win);
		}
		else wprintw(win, " %-18.18s ", item[c].GetName().c_str());
		
		waddch(win, VLINE);	
		waddch(win, ' ');
	}
	
	// -------
	wmove(win, item.size()+1, 0);
	waddch(win, ' ');
	waddch(win, LLCORNER);
	whline(win, HLINE, 20);
	mvwaddch(win, item.size()+1, 22, LRCORNER);
	waddch(win, ' ');
		
	unsetcol(win);
	
	// 
	setcol(7, 0, win);
	wmove(win, index+1, 2);
	
	wprintw(win, " %-18.18s ", item[index].GetName().c_str());
		
	unsetcol(win);
	
	wrefresh(win);
}

void MLS::MenuCategory::CursorUp( )
{
	if (item.empty()) return;
	
	index = (index +item.size() - 1) % item.size();
	
	// 빈 스트링은 건너뛴다	
	// 메뉴전체가 빈 스트링일 경우 무한 루프 빠질위험 있음
	if (item[index].GetName().empty()) CursorUp();	
}

void MLS::MenuCategory::CursorDown( )
{
	if (item.empty()) return;
	
	index = (index + 1) % item.size();
	
	// 빈 스트링은 건너뛴다	
	// 메뉴전체가 빈 스트링일 경우 무한 루프 빠질위험 있음
	if (item[index].GetName().empty()) CursorDown();
}
