#ifndef Menu_h_
#define Menu_h_

#include <string>
#include <vector>
#include <ncurses.h>

namespace MLS {

class Panel;

class MenuItem
{
	std::string name;
	std::string command;
	
public:
	
	MenuItem(const std::string &n, const std::string &c) 
		: name(n), command(c) 
	{}
	
	// setter	
	void SetName(const std::string &n) {name = n;}
	void SetCmd(const std::string &n) {command = n;}	
	
	// getter	
	const std::string &GetName() {return name;}
	const std::string &GetCmd() {return command;}		
};

// 메뉴의 분류
class MenuCategory
{
	std::string name;
	
	int index;     // 메뉴 내에서의 위치
	int location;
	
	std::vector<MenuItem> item; // 서브메뉴
	
	WINDOW *win;	
	
public:
	MenuCategory(const std::string &p, const int &l = 0)
		: name(p) , index(0), location(l), win(NULL)
	{}
	~MenuCategory()
	{
		if (win) delwin(win);
	}
			
	const std::string &GetName(){return name;}
	
	void AddItem(const std::string &n, const std::string &c)
	{
		item.push_back(MenuItem(n,c));				
	}
	
	void SetLocation(const int &x){ location = x;}
	int  GetLocation() { return location;}
	
	const std::string &GetCmd() { return item[index].GetCmd(); }	
	void Draw();
	void Create();
	
	void CursorUp();
	void CursorDown();
	
	
};

class Menu
{
protected:
	int num;	
	std::vector<MenuCategory> item;
	
	WINDOW *win;
	
public:
	Menu();
	~Menu();
	
	void AddItem(MenuCategory p)
	{
		if (item.empty()) p.SetLocation(4);
		else p.SetLocation(item.back().GetLocation() + item.back().GetName().size() + 2);
			
		item.push_back(p);
	}
	
	void AddItem(const std::string &p)
	{
		AddItem(MenuCategory(p));		
	}
	
	void Create();
	void CreateWindow();
	
	void Draw();	
	void Process(MLS::Panel &);

};

}; // namespace

#endif 
