#include <ncurses.h>
#include <map>
#include <sys/stat.h>

#include "mls.h"
#include "panel.h"
#include "mcd.h"
#include "conf.h"
#include "mlslocale.h"
#include "strutil.h"
#include "keydef.h"

#include "dialog.h"

using namespace strutil;
using namespace MLS;

Panel::Panel()
{
	cur=0;
	page=0;
	
	sel_size=0;
	sel_num=0;
	
	// Zip 정의
	m_nZip = 0;
	m_sZipFile = "";
	
	bExit=false;
	
	tMenu.Create();
}

Panel::~Panel()
{
}

int 
Panel::Search(const string &str)
{
	int l = file_list.size();
	
	string p=tolower(str);
	
	for (int t=0; t < l ; t++)
	{
		if ( file_list[t]->name.size() < str.size() ) continue;
		if ( tolower(file_list[t]->name.substr(0, str.size())) == p) return t;
	}	
	// 없으면
	return -1;	
}

void 
Panel::proc()
{
	int key, bkey;
	bool bSearch = false;
	
	string tTmp;
	string::size_type	tSizeType;
	
	while(!bExit)
	{
		draw();	

		key = getch();
		
		if (!bSearch && ( isalnum(key) || key == '.' ) ) bSearch = true;		
		
		if (bSearch &&  ( (32<= key && key <=126) || key == KEY_BS ) )
		{
			int tmp;
			// search mode;
			// 뒷부분에 key를 넣고
			
			if (key == KEY_BS) 
			{
				strSearch.erase(strSearch.size()-1, 1);
				if (strSearch.empty()) {bSearch = false; continue;}
			}
			else strSearch += (char) key;
			
			// strSearch로 시작하는 문자를 찾는다..						
			if ((tmp = Search(strSearch))!= -1) cur = tmp;
			else
			{
				strSearch = (char) key;
				// 없으면 처음에 넣고 다시 찾는다.
				
				if ((tmp = Search(strSearch))!= -1) cur = tmp;
				// 없으면 말구..
				else {bSearch = false;strSearch.clear();}
			}
			continue;
		}
		
		bSearch = false;
		strSearch.clear();
		
		if (KEY_F(1) <= key && key <=KEY_F(12))
		{
			if (conf.Func[key-KEY_F(1)].size()>=1)	Command((conf.Func[key-KEY_F(1)])[0]);
			continue;
		}
		
		switch(key)
		{
				
		case KEY_UP:
			if (cur > 0) cur--;
			break;
			
		case KEY_DOWN:
			if (cur < file_list.size()-1) cur++;
			break;
		
		case KEY_LEFT:
			if (cur < (unsigned)nrow) cur=0;
			else cur-=nrow;
			break;
		
		case KEY_RIGHT:
			cur+=nrow;
			if (cur > file_list.size()-1) cur=file_list.size()-1;
			break;
			
		case ' ':
			if (file_list[cur]->selected) deselect(*file_list[cur]);
			else select(*file_list[cur]);
			
			if (cur < file_list.size()-1) cur++;			
			break;
			
		case '*':
			for (pFileIterator i=file_list.begin();i!=file_list.end(); i++)
			{
				if ((*i)->selected) deselect(*(*i));
				else select(*(*i));
			}
			break;

		case KEY_HOME:
			cur = 0;
			break;
			
		case KEY_END:
			cur = file_list.size()-1;
			break;
		
		case KEY_PPAGE:
			if (cur < (unsigned)nrow*ncol) cur=0;
			else cur-=(nrow*ncol);
			break;
			
		case KEY_NPAGE:
			cur+=(nrow*ncol);
			if (cur > file_list.size()-1) cur=file_list.size()-1;
			break;
			
		case KEY_CTRL_P:
		{
			int size;
			vector<string> &p = GetSelection(size, true);
			
			for (vector<string>::iterator i=p.begin(); i!=p.end(); ++i)
				LOG("%s", (*i).c_str());
			break;
		}		
			
		case KEY_CTRL_T:
		{
			destroy();
			getch();
			init();
			break;
		}
		
		case KEY_CTRL_A:
		{
			vector<string> p;
			p.push_back("Yes");
			p.push_back("No");
			SelectBox("Mls SelectBox", p, 1);
						
			break;
		}
		
		case KEY_CTRL_X:
		{
			
			Command("tar xvzf %F --directory %D");
			break;
		}
					
		case KEY_BS:
			Read("..");
			break;
					
	
		/*case KEY_DC:
			if (!file_list[cur].bdir())
			{
				remove(file_list[cur].name.c_str());				
			}
			read(".");
			break;
		*/		
		
		case 13:
			if (m_nZip == 0)
			{
				LOG("Read :: %s", file_list[cur]->name.c_str());
				if (Read(file_list[cur]->name))
				{
					LOG("Read Success");
					break;
				}
			}
			else
			{
				if (file_list[cur]->FullName == "/")
				{
					// new된 Zip 을 삭제
					m_tReader->Destroy();
					m_nZip = 0;
					if (Read(GetCurrentPath()))
					{
						break;
					}
					break;
				}
				
				if (file_list[cur]->bDir)
				{
					// 압축 파일은 FullName 으로 한다.
					LOG("Read Zip :: %s", file_list[cur]->FullName.c_str());
					if (file_list[cur]->name == "..") // .. 구현
					{
						tTmp = file_list[cur]->FullName;
						tTmp = tTmp.substr(0, tTmp.size() - 1);
						
						tSizeType = tTmp.rfind("/");
						tTmp = tTmp.substr(0, tSizeType+1);
						
						m_zipPath = m_sZipFile + " :: /" + tTmp;
						if (Read(tTmp)) break;
					}
					else
					{
						m_zipPath = m_sZipFile + " :: /" + file_list[cur]->FullName;
						if (Read(file_list[cur]->FullName)) break;
					}
					break;
				}

				
			}
				
			if (!file_list[cur]->bDir)
			{
				WINDOW*	pWin = NULL;
				
				if (m_nZip == 0)
				{
					m_nZip = 1;
					pWin = MsgWaitBox("Wait", "Please wait !!!");
					
					m_sZipFile = file_list[cur]->name;
					
					m_zipPath = m_sZipFile + " :: /";
					if (Read("") == true)
					{
						if (pWin) MsgWaitEnd(pWin);
						break;
					}
				}
				else
				{
					break;
					/*
					LOG("Filename :: %s", file_list[cur]->FullName.c_str());
					
					if (Read(file_list[cur]->FullName) == true)
					{
						break;
					}
					else
					{
						if (pWin) MsgWaitEnd(pWin);
						MsgBox("ERROR", "Zip file reading fail!!!");
					}
					*/
				}
				
				m_nZip = 0;
				if (pWin) MsgWaitEnd(pWin);
					
				// 실행	
				std::map<std::string, Action>::iterator i	= conf.ext_prog.find( tolower( file_list[cur]->ext()) );
				
				if (i != conf.ext_prog.end())
				{
					if ((*i).second.size()==1)	Command(((*i).second)[0]);
				}
				else if (file_list[cur]->attr[3] == 'x')
				{
					std::string cmd;
					cmd = "./" + file_list[cur]->name;
					
					destroy();
					system(cmd.c_str());
					init();
				}
				break;
			}
			break;
			
		case '/':
		{
			string cmds;
			if (InputBox("Shell command", cmds)<0) break;
			Command(cmds);			
			break;
		}
			
		case '\\':
			Read("/");
			break;
			
		case 27: // ESC-meta, alt
			bkey = getch();
			
			switch(bkey)
			{
			case 27: // esc 두번 누르면 Zip은 종료 된다.
				if (m_nZip == 1)
				{	
					// new된 Zip 을 삭제
					m_tReader->Destroy();
					m_nZip = 0;
					if (Read(GetCurrentPath()))
					{
						break;
					}
				}
				break;
				
			case 'q':
				Read(tHistory.GetPrev());
				break;
				
			case 'w':
				Read(tHistory.GetNext());
				break;
				
			case '0':
				conf.AutoDrawmode = !conf.AutoDrawmode;
				break;
			case '1':
				conf.column = 1;
				conf.AutoDrawmode=false;
				break; 
			case '2':
				conf.column = 2;
				conf.AutoDrawmode=false;
				break;
			case '3':
				conf.column = 3;
				conf.AutoDrawmode=false;
				break;
			case '4':
				conf.column = 4;
				conf.AutoDrawmode=false;
				break;
			case '5':
				conf.column = 5;
				conf.AutoDrawmode=false;
				break;
			case '6':
				conf.column = 6;
				conf.AutoDrawmode=false;
				break;
				
			case 'N':
			case 'n':
				conf.sort = 1;
				Sort();
				break;
				
			case 'E':
			case 'e':
				conf.sort = 2;
				Sort();
				break;
				
			case 'S':
			case 's':
				conf.sort = 3;
				Sort();
				break;
				
			case 'T':
			case 't':
				conf.sort = 4;
				Sort();
				break;
			case 'B':
			case 'b':
				conf.sort = 5;
				Sort();
				break;
				
//----------------------------
			case 'C':
			case 'c':
				if (m_nZip == 0) cmd_copy();
				else cmd_zip_copy();
				break;
				
			case 'D':
			case 'd':
				// 압축 파일이 지워지면 다시 읽어들어야 되기 때문.
				if (m_nZip == 1) 
				{
					cmd_zip_remove();
					m_tReader->Destroy();
					
					m_nZip = 0;
					if (Read(GetCurrentPath()))
					{
						break;
					}
				}
				break;
				
			case 'k':
			case 'K':
				Command("<mkdir>");
				break;
				
			case 'R':
			case 'r':
				Command("<rename>");
				break;
			
				
			case 'V':
			case 'v':
				if (!file_list[cur]->bDir)
					Command(conf.extern_viewer + " %F");
					
				break;
			
			case 'X':			
			case 'x':
				bExit=true;
				break;
				
			case 'Z':
			case 'z':			
				conf.ShowHidden = !conf.ShowHidden;
				Refresh();
				break;
				
			case '`':
				set_locale(KO);
				break;
				
			default:			
				break;
			}
		}
	}
	
	string pathfile;
	
	pathfile = conf.cfghome + "path";
	
	FILE *fp= fopen(pathfile.c_str(), "wb");
	fputs(path.c_str(), fp);
	fclose(fp);
}
