#ifndef MLSmain_H__
#define MLSmain_H__

#include <list>
#include <vector>
#include <string>

#include "define.h"
#include "mcd.h"
#include "file.h"
#include "drawutil.h"
#include "strutil.h"
#include "mpool.h"
#include "reader.h"
#include "history.h"
#include "TargzUtil.h"
#include "menu.h"
#include "conf.h"

typedef unsigned int uint;

namespace MLS
{	
class Panel
{
protected:
	
	typedef std::vector<File *>::iterator pFileIterator;
	
	MemoryPool<File> pool;
	std::vector<File *> file_list;   // 지금 리스트에 나타낼 파일들.,
	
	History tHistory;
	Menu    tMenu;
		
	std::string path;  // path
	std::string strSearch;
	
	uint sel_size; // 선택한 것들 크기
	uint sel_num;  // 선택한 것들의 개수
			
	uint cur, page; // 커서
	int  ncol, nrow, npage;
	uint ndir, nfile, dir_size;
	
	bool bExit;
	bool bRoot;
	
	// Zip파일을 위한
	int		m_nZip;	
	string	m_zipPath;
	string	m_sZipFile;
	
	Reader *m_tReader;
	
	int CommandMessage(const std::string &);
	
public:
	Panel();
	~Panel();	
	void proc();
	
	// commands
	int Command(const std::string &);
	int Run(const std::string &, bool = false); // 실제 실행부분
	
	void Refresh();
	
	int about();	
	int cmd_help();
	int cmd_rename();
	int cmd_view();
	int cmd_edit();	
	int cmd_copy();
	
	int cmd_zip_copy(void);
	int cmd_zip_remove(void);
	
	int cmd_move();
	int cmd_remove();
	int cmd_menu();
	int cmd_attr();
	int cmd_mcd();
	
	// read	
	bool Read(const std::string &);
	
	void         Init();
	void         Sort();
	Reader		*GetReader(const std::string &path);	
	color_entry *GetColor(File &ext);
	
	const std::string& GetRealPath(const std::string &);
	const std::string& GetCurrentPath(void);
	
	// draw	
	void draw();
	void drawfile1();
	void drawfile2();
	void drawfile3();
	void drawfile4();
	void drawfile5();
	void drawfile6();
	
	
	
	
	// select
	void select(File &);
	void deselect(File &);	
	
	std::vector<std::string> &GetSelection(int &size, bool bDirExpansion = false);
	
	// search
	
	int  Search(const std::string &);	
	
protected:

class is_dir
{
public:
	bool operator()(File *a)
	{
		return a->bDir;
	}
};

class sort_color
{
public:
	bool operator()(File *a, File *b)
	{		
		if (a->color==b->color)	return a->name < b->name;
		
		if (*a->color == conf.Else) return false;
		if (*b->color == conf.Else) return true;
		
		return *a->color < *b->color;
	}

};

class sort_name
{
public:
	bool operator()(File *a, File *b)
	{
		return a->name < b->name;		
	}
};

class sort_ext
{
public:
	bool operator()(File *a, File *b)
	{
		if (a->ext()==b->ext())	return a->name < b->name;
		return a->ext() < b->ext();
	}
};

class sort_size
{
public:
	bool operator()(File *a, File *b)
	{
		if (a->size==b->size)	return a->name < b->name;
		return a->size < b->size;	
	}
};

class sort_time
{
public:
	bool operator()(File *a, File *b)
	{
		if (a->mtime==b->mtime)	return a->name < b->name;
		return a->mtime < b->mtime;
	}
};

};

}; // namespace

#endif
