#include <ncurses.h>
#include <sys/stat.h>

#include <string>
#include <vector>

#include "mls.h"
#include "panel.h"
#include "strutil.h"
#include "dialog.h"
#include "conf.h"
#include "StringTokenizer.h"

inline string addslash(const string &p)
{
	//  t
	// 01234
	// jones
	string arg;
	
	string::size_type t  = 0,
					  t2 = p.find_first_of(" ()&*?$'\"\\\0");
	
	while(t2 != string::npos)
	{
		arg.append(p, t, t2-t);
		arg += '\\';
		arg += p[t2];		
		t = t2 + 1;
		t2 = p.find_first_of(" ()&*?$'\"\\\0", t);
	}	
	arg.append(p, t, p.length()-t);
	
	return arg;			
}


using namespace std;
using namespace strutil;

using namespace MLS;

int MLS::Panel::CommandMessage(const std::string &c)
{// 이부분 커맨드가 많아진 관계로 나중에 맵핑처리할 것..
	
	if (c == "del")
	{
		return cmd_remove();
	}
	
	if (c == "editconfig")
	{
		Run(conf.extern_editor + " " + conf.home + ".mls/mls.cfg"); 
		
		conf.load((conf.home + ".mls/mls.cfg").c_str());
		Refresh();
		return 0;		
	}
	
	if (c == "editcolorset")
	{
		Run(conf.extern_editor + " " + conf.home + ".mls/" + conf.colorset);
		conf.load((conf.home + ".mls/mls.cfg").c_str());
		
		Refresh();
		
		return 0;
	}
	
	
	if (c == "about")
	{
		about();
		return 0;
	}
	
	if (c == "help")
	{
		cmd_help();
		return 0;				
	}
	
	if (c == "exit")
	{
		bExit=true;
		return 0;
	}
	
	if (c == "menu")
	{
		tMenu.Process(*this);
		return 0;		
	}
	
	if (c == "refresh")
	{
		Refresh();
		return 0;
	}
	
	if (c == "mcd")
	{
		Mcd mcd;
		mcd.AddDirectory(path);
		mcd.setCur(path);
		if (!mcd.proc()) return 1;
		
		Read(mcd.getCur());
		return 0;
	}
	
	if (c == "copy") 
	{
		if (m_nZip == 0)
			return cmd_copy();
		else 
			return cmd_zip_copy();		
	}
	
	if (c == "move") 
	{
		if (m_nZip == 0)
			return cmd_move();
		return 0;
			//return cmd_zip_copy();		
	}
	
	if (c == "edit")
	{
		return Run(conf.extern_editor + ' ' + addslash(file_list[cur]->name));
	}
	
	if (c == "view")
	{
		return Run(conf.extern_viewer + ' ' + addslash(file_list[cur]->name));
	}
	
	if (c == "mkdir")
	{
		string p;
		if (InputBox("Make Directory", p)<0) return 1;
		
		struct stat statbuf;
		
		if (stat(p.c_str(), &statbuf)!=-1)
		{
			MsgBox("Error","Directory exists");
			return 1;				
		}
		
		if (mkdir(p.c_str(), 0755)==-1)
		{
			MsgBox("Error","Fail to make a directory");
			return -1;
		}
		Refresh();
		return 0;	
		
	}
	if (c == "rename")
	{		
		struct stat statbuf;
		string ren = file_list[cur]->name;
		
		if (ren == "..") return 0;
		
		if (InputBox("rename", ren)<0) return 1;
		
		if (lstat(ren.c_str(), &statbuf) != -1)			
		{
			MsgBox("Error", "fail : name exists");
			return 0;
		}
		
		if (rename(file_list[cur]->name.c_str(), ren.c_str()) == -1)
		{
			MsgBox("Error", "fail to rename");
			return 0;
		}
			
		file_list[cur]->name=ren;
			
		Refresh();
		return 0;
	}		
	
	return 0;						
}

// 모든 커맨드 메시지를 관리
int Panel::Command(const string &p)
{
	if (p.empty()) return 0;	
		
	// Command
	if (chop(p)[0] == '<')
		return CommandMessage(tolower(getbetween(p, '<', '>')));	
	
	// 나머지	
	File &rf=*file_list[cur];
	bool bFlag=false, bPause=false, bParam=false, bMcd = false, bConfirm = false;	
	
	string arg;

	// 파싱
	for (string::const_iterator i = p.begin(); i!= p.end(); i++)
	{
		if (*i == '%')
		{
			bFlag = true;
			continue;
		}
		
		if (bFlag)
		{
			switch(*i)
			{
			// 파일을 리턴해줌
			case '1':			
			case 'F':
				arg += addslash(rf.name);
				break;
				
			case 'N':
			{
				if (rf.name[0]=='.') return 0;
				
				string::size_type p = rf.name.rfind('.');
				
				if (p!=string::npos)
					arg += addslash(rf.name.substr(0, p-1));
				else
					arg += addslash(rf.name);
				break;
			}			
			
			case 'E':
				if (rf.ext().empty()) return 0;
				arg += addslash(rf.ext());
				
				break;
				
			case 'W':
				bPause = true;
				break;
				
			case '%':
				arg += '%';			
				break;
				
			case 'S':
			{
				if (!sel_num) return 0;
				
				bool first = true;
				for (pFileIterator i = file_list.begin(); i!=file_list.end(); i++)
					if ((*i)->selected)
					{
						if (first) first=false;
						else arg+=' ';
						arg += addslash((*i)->name);
					}
				break;
			}
			
			case 's':
			{
				if (rf.name == "..") return 0;
				
				if (sel_num)
				{
					bool first = true;
					for (pFileIterator i = file_list.begin(); i!=file_list.end(); i++)
						if ((*i)->selected)
						{
							if (first) first=false;
							else arg+=' ';
							arg += addslash((*i)->name);
						}
				}
				else 
					arg += addslash(rf.name);
				
				break;
			}
			
			case 'd':
			
				arg += addslash(path);
				break;		
			
			case 'D':
				bMcd = true;				
				break;
			
			case 'P':
				bParam= true;
			
				break;
				
			case 'H':
				arg += addslash(conf.home);
				
				break;
				
			case 'Q':
				bConfirm = true;
				break;
				
			default:
				break;
			}
			
			bFlag = false;
			continue;
		}
		else arg += *i;
	}
	
	if (bMcd)
	{
		Mcd mcd;
		mcd.setCur(path);
		if (!mcd.proc()) return 0;
		arg += addslash(mcd.getCur());
	}
	
	if (bParam)	if (InputBox("Edit Parameter", arg)<0) return 0;
		
	if (bConfirm)
		if (YNBox("Do you want to run this operation?", YN_N) != YN_Y) return 0;
		
	// 실행
	
	return Run(arg, bPause);
}

int Panel::Run(const std::string &cmd, bool bPause)
{
	destroy();
	int r = system(cmd.c_str());
	
	if (bPause)
	{
		printf("\nPress any key to continue...");
		getchar();
	}
	init();
	
	Refresh();
	
	return r;	
}
