#include <string>
#include <sstream>
#include <sys/stat.h>
#include <errno.h>


#include "define.h"
#include "panel.h"
#include "mcd.h"
#include "dialog.h"
#include "strutil.h"

using namespace std;
using namespace strutil;

using namespace MLS;

// tar.gz 파일 압축 풀고 파일 복사.
int Panel::cmd_zip_copy(void)
{
	Mcd mcd("Select the directory where you want to copy these files");
	mcd.AddDirectory(path);
	mcd.setCur(path);
	if (!mcd.proc()) return -1;
	
	string msg = "To " + mcd.getCur();
	
	StatBox s("Copy", msg.c_str());	
	s.setUL("Calculating file sizes...");
	s.show();
	
	int size = 0;	
	vector<string> &unzip_files=GetSelection(size, false);
	
	string sTarget_path = mcd.getCur();
	LOG("MCD PATH :: %s", sTarget_path.c_str());
	
	if (m_tReader->Copy(unzip_files, sTarget_path) == ERROR)
	{
		return ERROR;
	}
	return SUCCESS;
}

int Panel::cmd_copy(void)
{
	Mcd mcd("Select the directory where you want to copy these files");
	
	mcd.AddDirectory(path);
	mcd.setCur(path);
	
	if (!mcd.proc()) return -1;
	
	StatBox s("Copy", ("To " + mcd.getCur()).c_str());	
	
	s.setUL("Calculating file sizes...");
	s.show();
		
	// 1. 선택한 것을 하는 것인가..
	// 2. 그냥 한 파일만 복사하는 것인가
	
	int size = 0;	
	vector<string> &files=GetSelection(size, true);
	string sizes = toregular(size), rest_size;	
	
	struct stat src_stat, tar_stat, dir_stat;
	
	FILE *fp, *out;	 // 원본, 타겟
	ostringstream count;
	
	char *buf = new char[65536]; 
	
	uint last_size, p=1, p_size=0, c_size;
	
	int key;		
	
	// /home/ioklo/ 에서 복사 시작  /p/로..
	// /home/ioklo/mls/p.dir   ==> /p/mls/p.dir
	
	// /home/ioklo/ 현재 디렉토리 source_path
	// /p/          타겟 디렉토리 target_path
	// mls/         복사시점 디렉토리 cur_path
	// p.dir        파일 이름  cur_name
	// /p/mls/      타겟 복사시점디렉토리..  tcur_path = target_path + cur_path
	
	// mls/p.dir    = cur_path + cur_name
	// /p/mls/p.dir = tcur_path + cur_name
	
	string target_path = mcd.getCur();
		
	string source_name;
	string target_name;
	
	string cur_path = path;	
	string tcur_path = target_path;
	
	bool bOverwriteAll = false;
	bool bsDir=true;	
	
	// 파일 복사
	for (vector<string>::iterator i=files.begin(); i!=files.end(); ++i)
	{
		string &cur_name = *i;
		
		LOG("%s", cur_name.c_str());
		
		// 파일 이름이 없을경우 continue	
		if (cur_name.empty()) continue;	
		
		// 파일 리스트가 디렉토리 라면
		if (cur_name[cur_name.size()-1]=='/') 
		{
			// 타겟 디렉토리가 있는지 검사한다..
			bsDir = true;	
			
			// /p/mls/
			 cur_path = path + cur_name;
			tcur_path = target_path + cur_name;		
			
			if (stat(tcur_path.c_str(), &dir_stat) != -1)
			{// 이미 디렉토리가 있다면
				if (YNBox("Directory or file Exists, continue?", YN_N) == YN_N)	break;
				continue;
			}
			// 디렉토리를 만든다.
			else if (mkdir(tcur_path.c_str(), 0755)==-1)
			{// 실패하면..				
				MsgBox("Error", "Directory Making error");
				break;
			}
			continue;
		}
		// 이번 디렉토리가 실패이면.. 다음 디렉토리때까지  복사하지 않는다.
		if (!bsDir) continue;
		
		// 소스 이름과 타겟 이름 정하기	
		source_name =  cur_path + cur_name;
		target_name = tcur_path + cur_name;
					
		if (stat(source_name.c_str(), &src_stat)==-1)
		{
			// 소스가 없다니.. 참
			MsgBox(COLOR_RED, "Error", "Source is not found : %s", source_name.c_str());
			break;
		}
		
		mode_t permission = src_stat.st_mode;
		
		count.str("");
		count << '<' << p << '/' << files.size() << '>';
		
		s.setUL((*i).c_str());
		
		s.setBL(count.str().c_str());		
		s.setBR(sizes.c_str());	
		s.show();	
		
		if (stat(target_name.c_str(), &tar_stat)!=-1 && bOverwriteAll == false)
		{// 파일이 있을 경우..			
			vector<string> q;			
				q.push_back("Overwrite");
				q.push_back("Skip");
				q.push_back("Overwrite All");
				q.push_back("Cancel");
			
			int x = SelectBox(("File exists : " + (*i)).c_str(), q, 0);
			
			s.redraw();
			
			switch(x)
			{
			case 0:				
				break;
				
			case 1:
				p_size += src_stat.st_size;
				p++;
				continue;
				
			case 2:
				bOverwriteAll = true;
				break;
						
			case 3:
				goto halt;
						
			default:
				break;
			}
		}
		
		c_size=0;	
		// 실제 복사 오퍼레이션	
		
		p++;
		
		if (src_stat.st_ino == tar_stat.st_ino) continue;		
		
		fp = fopen(source_name.c_str(), "rb");
		out = fopen(target_name.c_str(), "wb");
		
		if (fp && out)	
		{
			nodelay(stdscr, TRUE); // esc키 처리//
			
			while( !feof(fp) )
			{
				key=getch();
							
				if (key!=-1)
				{
					nodelay(stdscr, FALSE);
					if (YNBox("Do you want to stop copy operation?", YN_N)==YN_Y)
						goto halt;					
					s.redraw();
				}				
				
				last_size = fread(buf, 1, 65536, fp);			
				fwrite(buf, 1, last_size, out);
				
				c_size +=last_size;
				p_size +=last_size;
				
				rest_size = toregular(p_size);
				
				s.setUR(rest_size.c_str());
				
				if (src_stat.st_size) 
					s.setU(c_size * 60 / src_stat.st_size);
				
				if (size) 
					s.setB(p_size * 60 / size);
				
				s.show();
			}
			nodelay(stdscr, FALSE); //다시 원상복구
		}
		else
		{
			if (YNBox("File access error, continue?", YN_N) == YN_N) break;
		}
		
		fclose(out);
		fclose(fp);
		
		chmod(target_name.c_str(), permission);			
	}
halt:
	
	delete []buf;
	return 1;
}
