#include <dirent.h>
#include <sys/stat.h>
#include <time.h>
#include <string>
#include <map>

#include <unistd.h>

#include "panel.h"
#include "conf.h"
#include "strutil.h"
#include "dialog.h"
#include "ZipReader.h"

using namespace std;
using namespace strutil;
using namespace MLS;

// read interface
/*
 * (기본) 파일 리더.
 * file:// 파일 리더
 * arc:// 
 * ftp:// 
 */
 
void Panel::Init(void)
{
	cur      = 0;
	page     = 0;
	sel_size = 0;
	sel_num  = 0;	

	nfile    = 0;
	ndir     = 0;
	dir_size = 0;	
	
	file_list.clear();
	pool.Clear();	
}

color_entry *Panel::GetColor(File &t)
{
	// 이름
	map<string, color_entry>::iterator namei = conf.name_col.find( tolower(t.name) );
	
	if (namei != conf.name_col.end() ) 
		return &(*namei).second;
	
	// 퍼미션
	int mode=0;
	
	for (int z=1; z<10; z++)
	{
		mode <<= 1;
		mode +=	t.attr[z] == '-' ? 0 : 1; 		
	}	
	for (map<int , color_entry>::iterator mi = conf.mask_col.begin(); mi != conf.mask_col.end(); ++mi)
		if (mode & (*mi).first) return &(*mi).second;				
	
	
	// 확장자
	map<string, color_entry>::iterator exti = conf.ext_col.find( tolower(t.ext()) );
	
	if (exti != conf.ext_col.end() ) 
		return &(*exti).second;
	
	return &conf.Else;	
}

void Panel::Sort(void)
{
	// 소팅을 시작합니다	
	if (file_list.size() > 1)
	{	
		pFileIterator i, 
			 begin = file_list.begin(),
			 end   = file_list.end();
		
		if (conf.DirSort)
		{
			if (!bRoot) 
			{// ..을 찾아서 맨 앞으로(루트는 없다)
				for(i = begin; i != end; i++)
					if ((*i)->name == "..")
					{
						File *tmp = *begin;
						*begin = *i;
						*i = tmp;
						begin++;
						break;
					}
			}
			
			// 디렉토리들을 찾아서 앞으로..
			pFileIterator s=end-1;
			
			for (;begin!=end; begin++)
			{
				if (!(*begin)->bDir)
				{
					while(s != begin && !(*s)->bDir) s--;
					
					if (begin == s) break;
					
					File *tmp   = *begin;
					     *begin = *s;
					     *s     = tmp;	
				}
			}
			
			if (bRoot) sort(file_list.begin(), begin, sort_name());
			else sort(file_list.begin()+1, begin, sort_name());
		}	
		
		// 정렬	
		switch(conf.sort)
		{
		case 1: // name
			sort(begin, end, sort_name());	
			break;
		case 2: // ext
			sort(begin, end, sort_ext());	
			break;
		case 3: // size
			sort(begin, end, sort_size());	
			break;	
		case 4: // time
			sort(begin, end, sort_time());	
			break;
		case 5:  // color
			sort(begin, end, sort_color());	
			break;			
		case 0:
		default:
			break;
		}
		
	}	
}

void Panel::Refresh(void)
{
	int tmp;
	string cur_name = file_list[cur]->name;
	
	Read(".");
		
	if ((tmp = Search(cur_name)) != -1) cur= tmp;
}

const string& Panel::GetCurrentPath(void)
{
	static string s_path;
	
	char *tmpdir  = getcwd(NULL, 0);
	s_path = tmpdir;	
	free(tmpdir);
	
	if (s_path != "/") s_path += '/';
	
	return s_path;
}

const string &Panel::GetRealPath(const string &str)
{
	static string s_path;
	
	s_path = str;
	
	if (m_nZip == 0)
	{
		if (s_path[0]!='/')
		{
			if (s_path == ".") s_path = path.empty() ? GetCurrentPath() : path;
			else if (s_path == "..")
			{
				if (path != "/")
				{
					string::size_type p = path.rfind('/', path.size()-2);
					s_path = path.substr(0, p+1);
				}			
				else s_path = path;
			}
			else s_path = path + s_path + '/';
		}
	}
	else
	{
		s_path = GetCurrentPath();
	}
	return s_path;
}

//  path 뒤에는 항상 /가 붙는 것을 원칙으로 합니다. (예 /home/iddd/ )

bool Panel::Read(const string &dir_name)
{
	string prev_dir;
	string strPath  = "";
	
	if (dir_name == ".."  && path != "/")
	{
		string::size_type p = path.rfind('/', path.size()-2);
		prev_dir = path.substr(p+1, path.size()-p-2);
	}
	
	strPath  = GetRealPath(dir_name);		// 상대경로를 절대경로로 바꾼다.
	
	//MsgBox("Error", strPath.c_str());
	m_tReader = GetReader(strPath);			// 절대경로에 맞는 Reader를 얻어온다	
	
	if (m_tReader == NULL) return false;
	
	if (m_nZip == 0)	
	{
		if (!m_tReader->Init(strPath)) 		return false; // Init 은 항상 절대경로로.		
	}
	else
	{
		LOG("Zip File INIT :: %s", dir_name.c_str());
		
		if (!m_tReader->Init(dir_name)) // Zip 은 dir_name으로	
		{
			//MsgBox("ERROR", "Zip file reading failure!!!");
			LOG("zip file read fail.."); 
			return false;
		}
	}
		
	bRoot = m_tReader->isRoot();
	
	path  = strPath;
	
	tHistory.AddEntry(path);
	
	Init(); //panel init
	
	while(m_tReader->Next())
	{
		File &t = pool.Get();
		
		if (!m_tReader->GetInfo(t)) continue;
		
		if (m_nZip == 0)
		{
			if (t.name == ".")	continue;
			if (bRoot && t.name == "..") continue;
			if (!conf.ShowHidden && t.name[0] == '.' && t.name != ".." ) continue;
		}
		else
		{
			//if (!conf.ShowHidden && t.name[0] == '.' && t.name != ".." ) continue;
		}
		
		dir_size += t.size;
		
		t.selected = false;
		t.color    = GetColor(t);
		
//		LOG("FileList Data :: %s", t.name.c_str());
		file_list.push_back(&t);
		
		if (t.bDir) ndir++;
		else nfile++;
	}
	
	Sort();	
	
	if (!prev_dir.empty()) // ..에 해당하는 디렉토리를 찾는다..
	{
		int tmp;
		if ((tmp = Search(prev_dir)) != -1) cur = tmp;
	}
	
	return true;
}
